<?php

namespace Drupal\better_json_response;

use Drupal\Core\Cache\CacheableResponseInterface;
use Drupal\Core\Cache\CacheableResponseTrait;
use Symfony\Component\HttpFoundation\JsonResponse as BaseJsonResponse;

/**
 * Improved JsonResponse with Json:API support and cache.
 */
class BetterJsonResponse extends BaseJsonResponse implements CacheableResponseInterface {

  use CacheableResponseTrait;

  /**
   * Original data before processing to json.
   *
   * @var mixed|null
   */
  protected mixed $originalData;

  /**
   * {@inheritDoc}
   */
  public function __construct(mixed $data = NULL, int $status = 200, array $headers = [], bool $json = FALSE) {
    $this->originalData = $data;
    parent::__construct($data, $status, $headers, $json);
  }

  /**
   * Return originalData property value.
   *
   * @return mixed|null
   *   Value of originalData property.
   */
  public function getOriginalData(): mixed {
    return $this->originalData;
  }

  /**
   * {@inheritDoc}
   */
  public function setData(mixed $data = []): static {
    $this->originalData = $data;
    return parent::setData($data);
  }

  /**
   * Determine if cacheability is defined.
   *
   * @return bool
   *   TRUE if cacheability is defined in response.
   */
  public function isCacheabilityDefined(): bool {
    return isset($this->cacheabilityMetadata);
  }

}
